local T, scan_sector = ...

function T:BuildDisplay()
	local num_width, health_width, name_width, distance_width
	local fontsize = Font.Default * HUD_SCALE * T.fontscale

	local function display_row_width() return num_width + health_width + name_width + distance_width end


	local function build_alt_tab_display(num, init_tbl)

		local line_width = health_width + name_width + distance_width
		local sum_tab_width_pct = 0 -- current sum of column percents.  Should not exceed 1.

		local font = init_tbl.font and init_tbl.font * HUD_SCALE * T.fontscale or fontsize

		local row_display_elements = {iup.label{title = num, size = num_width, font = font, expand = "NO"}, gap = 5}

		for i = 1, 3 do
			local ctrl
			local col = init_tbl.columns and init_tbl.columns[i] or {}
			-- verify and adjust column percent as necessary
			local w = col.width or 0
			w = (w > 1 and 1) or (w < 0 and 0) or w
			w = (sum_tab_width_pct + w) > 1 and (1 - sum_tab_width_pct) or w
			w = i == 3 and (sum_tab_width_pct + w) < 1 and (1 - sum_tab_width_pct) or w
			sum_tab_width_pct = sum_tab_width_pct + w

			if ( w == 0) then
				ctrl = iup.fill{size = 0}
			else
				ctrl = iup.text{value = col.title or "", size = line_width * w, font = font, active = "NO", border = "NO", bgcolor = "0 0 0 255"}
			end
			table.insert(row_display_elements, ctrl)
		end
		return iup.hbox(row_display_elements)
	end
		
	
	local function build_ship_display(num, health, name, distance)
		num = num or ""
		health = health or ""
		name = name or ""
		name_width = name_width or self.name_width
		distance = distance or ""

		num = iup.label{title = num, size = num_width, font = fontsize, expand = "NO"}
		health = iup.label{title = health, size = health_width, font = fontsize, expand = "NO"}
		name = iup.text{value = name, size = name_width, font = fontsize, expand = "NO", active = "NO", border = "NO", bgcolor = "0 0 0 255"}
		distance = iup.label{title = distance, size = distance_width, font = fontsize, alignment = "ARIGHT", expand = "NO"}
		
		local shield = iup.stationprogressbar{
			visible = "NO", active = "NO", expand = "HORIZONTAL", title = "",
			size = "x"..math.ceil(fontsize/4),
			minvalue = 0, maxvalue = 100, uppercolor = "88 88 88 255 *", lowercolor = "65 65 255 255 *",
			value = 0, uv = "0 0.25 1 0.75"
		}

		local display = iup.hbox{
			num,
			health,
			iup.zbox{
				iup.vbox{
					iup.fill{size = math.ceil(fontsize*3/5)},
					shield,
				},
				name,
				all = "YES",
			},
			distance,
			gap = 5,
		}

		if (not health_width) then
			num:map();      num_width = tonumber(num.w) -- .size:match("(%d+)x"))
			health:map();   health_width = tonumber(health.w) -- .size:match("(%d+)x"))
			name:map();     name_width = name_width or tonumber(name.w) -- .size:match("(%d+)x"))
			distance:map(); distance_width = tonumber(distance.w) -- .size:match("(%d+)x"))
			num.title = ""
			name.value = name.value:gsub("W", "")
			name.size = name_width
		end
		return display
	end


	-- Always defaults to tab 0 (Targets).  If it's another tab, then we need to change the build func and second argument to pass the necessary data
	local build_func = build_ship_display
	local second_arg = "Health"
	if (T.tab_index > 0) then
		-- get the target column widths measured again
		build_func(99, second_arg, "Name"..string.rep("W", 10), "Distance")
		build_func = build_alt_tab_display
		second_arg = T.tab_identifiers[T.tab_index + 1]
	end


		-- depending on the display, I need to be able to send the init function into the builder
	local display_elements = {build_func(99, second_arg, "Name"..string.rep("W", 10), "Distance"),gap = 0}
	display_elements[1][1].visible = "NO" -- hide the number indicator in the header row
	for i = 1, self.list_count do

		local display_item = iup.zbox{
			iup.label{title = "", size = "", expand = "YES", bgcolor = "0 0 0 0 *"}, -- row highlight
			build_func(i, second_arg), -- only i is necessary for ship display.
			alignment = "CENTER",
			all = "YES",
		}
		table.insert(display_elements, display_item)
		display_elements[i + 1].visible = "NO" -- hide non-header rows
	end

	self.display = iup.vbox(display_elements)

	local filter_height_adjust = fontsize > 13 and 4/5 or 1
	local filter_item_width = display_row_width() / 4

	self.filters = iup.hbox{
		iup.label{title = "aaa", visible = "NO", font = fontsize, alignment = "ACENTER", expand = "YES", size = filter_item_width.."x"..(fontsize*filter_height_adjust)},
		iup.label{title = "bbb", visible = "NO", font = fontsize, alignment = "ACENTER", expand = "YES", size = filter_item_width.."x"..(fontsize*filter_height_adjust)},
		iup.label{title = "ccc", visible = "NO", font = fontsize, alignment = "ACENTER", expand = "YES", size = filter_item_width.."x"..(fontsize*filter_height_adjust)},
		iup.label{title = "ddd", visible = "NO", font = fontsize, alignment = "ACENTER", expand = "YES", size = filter_item_width.."x"..(fontsize*filter_height_adjust)},
	}
	local filter_box = iup.zbox{
		self.filters,
		iup.label{title = "", font = fontsize, size = "x"..(fontsize*filter_height_adjust), expand = "YES", bgcolor = "255 0 0 "..(T.invert_faction_filter and 35 or 0)},
		margin = "0x0",
		all = "YES",
		--visible = self.tab_index == 0 and "YES" or "NO"
	}

	self.tabs = iup.hbox{
		iup.label{title = "aaa", visible = "NO", font = fontsize, alignment = "ACENTER", expand = "YES", size = filter_item_width.."x"..(fontsize*filter_height_adjust)},
		iup.label{title = "bbb", visible = "NO", font = fontsize, alignment = "ACENTER", expand = "YES", size = filter_item_width.."x"..(fontsize*filter_height_adjust)},
		iup.label{title = "ccc", visible = "NO", font = fontsize, alignment = "ACENTER", expand = "YES", size = filter_item_width.."x"..(fontsize*filter_height_adjust)},
		iup.label{title = "ddd", visible = "NO", font = fontsize, alignment = "ACENTER", expand = "YES", size = filter_item_width.."x"..(fontsize*filter_height_adjust)},
		margin = "0x0",
	}
	local tab_box = self.tabs

	self.display_container = iup.pdarootframebg{ -- vboxes have fullscreen size.  A frame or other drawn control is necessary for sizing various things
		iup.vbox{
			(self.tabs_number_of > 1 and tab_box or iup.fill{}),
			iup.hudrightframe{
				iup.vbox{
					self.display,
					filter_box,
				},
			},
		},
		cx = 0, --self.x or (HUD.selfinfoframe.x - display_row_width() - 50),
		cy = 0, --self.y or (HUD.groupinfoframe.y),
		expand = "NO",
		bgcolor = "0 0 0 0 *", -- hide the containing frame
	}

	function self.display_container:PositionDisplay(x, y)
		x = tonumber(x) or (HUD.selfinfoframe.x - display_row_width() - 50)
		y = tonumber(y) or (HUD.groupinfoframe.y)
		self.cx = x
		self.cy = y
		HUD.cboxlayer.REFRESH = "YES"
		return x, y -- return x, y in case it was reset to default
	end

	local function update_target_list(i, list)
		local health, shield, name = "", "", ""
		local charid = list[i].charid
		local nodeid = GetPlayerNodeID(charid)

		name = list[i].name
		list[i].nodeid = nodeid
		list[i].objectid = GetPrimaryShipIDOfPlayer(charid)
		health, shield = GetPlayerHealth(charid)
		health = math.ceil(health or 0)
		local guild = GetGuildTag(charid)
		if (guild and guild ~= "") then name = "["..tostring(guild).."] "..name end

		local row = self.display[i + T.list_offset][2]
		row[2].title         = health == -1 and "" or health.."%"
		row[2].fgcolor       = calc_health_color(health * 0.01)
		row[3][1][2].visible = shield and "YES" or "NO"
		row[3][2].value      = name
		row[3][2].caret      = 1
		row[3][2].fgcolor    = FactionColor_RGB[list[i].factionid]
		row[4].title         = list[i].distance == 9999 and "" or list[i].distance

		if (shield) then
			shield = math.ceil(shield)
			row[3][1][2].value = shield
		end

		if (nodeid == self.current_target_nodeid) then
			self.display[i + T.list_offset][1].bgcolor = self.current_target_color
			self.current_target_listindex = i
		else
			self.display[i + T.list_offset][1].bgcolor = "0 0 0 0 *"
		end
	end

	-- Update the display of non-Target Tabs
	local function update_alt_tab(i, list)
		local row = self.display[i + T.list_offset]
		local list_item = list[i] or {}
		-- iterate over the three columns for the list display
		for j = 1, 3 do
			local v = list_item[j] or ""
			-- first column is the target number.  +1 to avoid it.
			row[2][j + 1].value = v
			row[2][j + 1].caret = 1
		end

		-- set the highlight if necessary
		if (list_item.highlight) then
			row[1].bgcolor = self.current_target_color
			self.current_target_listindex = i
		else
			row[1].bgcolor = "0 0 0 0 *"
		end
	end
	
	-- Update the targets list
	function self.display_container.update_display(list, alt)
		T.tab_target_list = list
		self.current_target_listindex = nil
		local display_func = alt and update_alt_tab or update_target_list

		for i = 1, self.list_count do
			
			if (list[i]) then

				display_func(i, list)
				self.display[i + T.list_offset].visible = "YES"
			else
				self.display[i + T.list_offset].visible = "NO"
			end
		end
	end

	local function set_selector_highlight(self, index, custom_fgcolor, custom_bgcolor)
		local fgcolor = custom_fgcolor or custom_bgcolor == 0 and "40 40 40 255 *" or "255 255 255 255 *"
		local bgcolor = FactionColor_RGB[custom_bgcolor] or custom_bgcolor or "0 0 0 0 *"
		self[index].bgcolor = bgcolor
		self[index].fgcolor = fgcolor
	end
	self.filters.SetHighlight = set_selector_highlight
	self.tabs.SetHighlight = set_selector_highlight

	function self.filters:GetFilterColors(tab_index) -- tab_index is already incremented by the caller.
		local findex, fpage = T.tab_filters:GetFilterPositions(tab_index)
		local filter = T.tab_filters[tab_index]
		local tab_filters = T.tab_identifiers[tab_index].filters
		local pages = tab_filters and tab_filters[findex + 1] and tab_filters[findex + 1].pages

		local fgcolor = pages and pages[fpage + 1].fgcolor or   -- First check if the filter page has a custom color
		                tab_filters and tab_filters[findex + 1].fgcolor -- Next check if the filter has a custom color

		local bgcolor = pages and pages[fpage + 1].bgcolor or   -- First check if the filter page has a custom color
		                tab_filters and tab_filters[findex + 1].bgcolor or -- Next check if the filter has a custom color
		                T.tab_index == 0 and fpage or           -- Check if it's the first tab and use the filter page to determine color from FactionColor_RGB by index
		                0                                             -- default to 0 index for FactionColor_RGB (white)

		return fgcolor, bgcolor
	end

	-- Build the tab display
	table.sort(T.tab_identifiers, function(a, b)
		if (a == T.Tab) then return true end
		if (b == T.Tab) then return false end
		return a.title < b.title
	end)

	for i, ident in ipairs(T.tab_identifiers) do
		self.tabs[i].title = ident.title
		self.tabs[i].visible = "YES"
		self.tabs[i].fgcolor = ident.fgcolor -- Display the custom fgcolor even when tab not selected.
	end

	-- Build the filters for the current tab
	for i, filter in ipairs(T.tab_identifiers[T.tab_index + 1].filters or {}) do
		if (i > 4) then break end
		self.filters[i].title = filter.title
		self.filters[i].visible = "YES"
	end
	

	local filter = self.tab_filters[self.tab_index + 1]
	local init_tbl = self.tab_identifiers[self.tab_index + 1]
	local pages = init_tbl.filters and init_tbl.filters[filter.index + 1].pages

	self.filters:SetHighlight(filter.index + 1, self.filters:GetFilterColors(self.tab_index + 1))
	self.tabs:SetHighlight(self.tab_index + 1, init_tbl.fgcolor, init_tbl.bgcolor or 0)

	iup.Append(iup.GetNextChild(HUD.cboxlayer), T.display_container)
	HUD.cboxlayer:map()
	self.display_container:PositionDisplay(T.x, T.y)

end

function T:DestroyDisplay()
	T.refresh_timer:Kill()
	iup.Detach(T.display_container)
	iup.Destroy(T.display_container)
	T.display_container = nil
	T.display = nil
	T.filters = nil
	T.tabs = nil
	self:DestroyRegions()
end

function T:RebuildDisplay()
	-- Reset the filter positions between tab shifts
	self:DestroyDisplay()
	self:BuildDisplay()
	if (self.tab_index == 0) then
		self.display[1][3][1][2].visible = "NO" -- Re-hide the progressbar in the header.  For some reason it doesn't hide properly on it's own.
	end
	self:CreateRegions()
	if (T.tab_index == 0) then scan_sector() end  -- can this be moved?
end
